/************************************************************************

	Atmega8 MOS6581 SID Emulator

	SID = Sound Interface Device

	This program tries to emulate the sound chip SID of the famous 
	historical C64 Commodore computer.
	The SID emulator includes all registers of the original SID, but 
	some funcitons may not be implemented yet.
	If you want	to program the SID registers to generate your own sound,
	please refer to the MOS6581 datasheet. The emulator tries to be as
	compatible as possible.
	
	This program consists of two parts: 
	The main program and the the include file sound.c .
	In the include file sound.c you can simply write down the register values
	to control the SID and let it playing some sound or music.
	The values are then interpreted by the emulator. There are also
	some instructions like pause and stop that are used for sound timing.

	In the main program there is an interrupt routine which calculates the 
	ouptut values for the PWM-Output every second time. 
	The reason for this is to achieve a high PWM rate to avoid audible artefacts
	at the PWM output.
	The output ist calculated every second time to save processing cycles.
	
	The cylce time in the main loop is 1ms. At this rate the envelope 
	generators are updated and the scheduler is called. 

	The actual amplitude value ist ouput at PIN15 as 8Bit PWM value. The
	sampling frequency is ~31Khz, therefore the high frequency noise
	of the PWM should not be audible to normal people.
	The PWM-Output may be directly connected to an audio amplifier.
	
************************************************************************

	Hardware
	
	prozessor:	ATMEGA8, ATMEGA168 ( see #define )
	clock:		16MHz Crystal

	PIN15	PB1/OC1A		8Bit PWM sound output
	PIN17	PB3/MOSI/OC2	8Mhz clock output, connect this pin to pin11
	PIN11	PD5/T1			8Mhz clock source for PWM
	PIN2	RXD				serial comunication input
	PIN19	PB0				test LED ( till version 1.7 Pin 14 was used )

****************************************************************************

	date	authors					version		comment
	======	======================	=======		==============================
	Apr.07	(ch) Christoph Haberer	V1.0		First implemetation
	Apr.22	(ch) Christoph Haberer	V1.2		some improvements
	Apr.25	(ch) Christoph Haberer	V1.4		ATMEGA168 added
	Jul.20	(ch) Christoph Haberer	V1.6		serial comunication
	Jul.28	(ch) Christoph Haberer	V1.7		some bugs removed
	Sep.28	(ch) Christoph Haberer	V1.8		additional gates
	
	Versions:

	V1.0
	- 3 waveform generators
	- 3 envelope generators with attack,decay, sustain, release
	- waverforms: triangle, rectangle with variable pulse width,
	  sawtooth, noise with variable frequency
	- ring modulation 
	- not implemented yet: prgrammable digital filter

	V1.2
	- some errors in the envelope generators removed
	- sustain levels initialized to maximum =0x0F
	- direct output/filter select bits implemented
	- short delay comand "0xF3,time[ms]," included in scheduler 
	V1.4 
	- ATMEGA168 added

	V1.6

	- serial comunication added, 9600 Baud, 
	first Byte: adress, second byte register value

	V1.8

	- LED Pin changed to from Pin 14 to Pin19 to be Arduino compatibel
	The Arduino has a dedicated LED resistor at PIN19
	3 additional LEDs to show the state of the gates

***************************************************************************/
/***************************************************************************
 *   
 *   (c) 2007 Christoph Haberer, christoph(at)roboterclub-freiburg.de
 *
 ***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation version 2 of the License,                *
 *   If you extend the program please maintain the list of authors.        *
 *   If you want to use this software for commercial purposes and you      *
 *   don't want to make it open source, please contact the authors for     *
 *   licensing.                                                            *
 ***************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
//#include <avr/signal.h>
#include <stdlib.h>
//#include "IIR2FILTER.h"
//#include "sound.h"

#define ATMEGA168
//#define ATMEGA8

//#define DEBUG
//#define NOISEOFF


#include "sound.c"

#define  FALSE	0
#define  TRUE	1

// CPU clock 16MHz
//#define F_CPU	16000000

#define FINT (F_CPU/512)	// 512=256PWM steps / 0.5 step per PWM interrupt
// FINT = 31250Hz @F_CPU16MHz

#define FS (FINT/2)
// FS = 15625Hz @F_CPU16MHz

#define MAXTIME 6000
#define OSCILLATORS 3

struct IIR_filter* filter04_06;

uint16_t freq_coefficient[OSCILLATORS];

uint16_t mstimer=0;

// SID registers

struct Voice
{
	uint16_t	Freq;			// Frequency: FreqLo/FreqHi
	uint16_t	PW;				// PulseWitdht: PW LO/HI only 12 bits used in SID
	uint8_t		ControlReg;		// NOISE,RECTANGLE,SAWTOOTH,TRIANGLE,TEST,RINGMOD,SYNC,GATE
	uint8_t		AttackDecay;	// bit0-3 decay, bit4-7 attack
	uint8_t		SustainRelease;	// bit0-3 release, bit4-7 sustain
};

struct Blocks
{
	struct Voice voice1;
	struct Voice voice2;
	struct Voice voice3;
	uint16_t FC;		// not implemented
	uint8_t RES_Filt;	// partly implemented
	uint8_t Mode_Vol;	// partly implemented
	uint8_t POTX;		// not implemented
	uint8_t POTY;		// not implemented
	uint8_t OSC3_Random;// not implemented
	uint8_t ENV3;		// not implemented
};

#define NUMREGISTERS 29

union Soundprocessor
{
	struct Blocks block;
	uint8_t sidregister[NUMREGISTERS];
} Sid;


// SID Registers
#define VOICE1	0
#define VOICE2	7
#define VOICE3	14
#define CONTROLREG 4
#define ATTACKDECAY 5
#define SUSTAINRELEASE 6

// SID voice controll register bits
#define GATE (1<<0)
#define SYNC (1<<1)		// not implemented
#define RINGMOD (1<<2)	// implemented
#define TEST (1<<3)		// not implemented
#define TRIANGLE (1<<4)
#define SAWTOOTH (1<<5)
#define RECTANGLE (1<<6)
#define NOISE (1<<7)

// SID RES/FILT ( reg.23 )
#define FILT1 (1<<0)
#define FILT2 (1<<1)
#define FILT3 (1<<2)
// SID MODE/VOL ( reg.24 )  
#define VOICE3OFF (1<<7)

// attack, decay, release envelope timings
uint16_t AttackRate[16]={2,4,16,24,38,58,68,80,100,250,500,800,1000,3000,5000,8000};
uint16_t DecayReleaseRate[16]={6,24,48,72,114,168,204,240,300,750,1500,2400,3000,9000,15000,24000};

// oszillator waveform arrays 
int8_t wave0[256];
int8_t wave1[256];
int8_t wave2[256];

// envelope arrays

uint8_t envelope[OSCILLATORS];

uint16_t 	m_attack[OSCILLATORS];
uint16_t	m_decay[OSCILLATORS];
uint16_t	m_release[OSCILLATORS];

uint8_t		release_flag[OSCILLATORS];
uint8_t		attackdecay_flag[OSCILLATORS];


int16_t	level_sustain[OSCILLATORS];
//int test[9];
//struct IIR_filter filter04_06;
// = {0,0,0,0, B0, B1, B2, A1, A2};          //initialize filter with 0.4 cutoff frequency

//uint16_t	t_attack[OSCILLATORS];
//uint16_t	t_decay[OSCILLATORS];	
//uint16_t	t_release[OSCILLATORS];	

uint8_t Serial_receiver_counter=0;
uint8_t Serial_receiver_bytes[2];

#define DEBUGPIN_OFF PORTD&=~(1 << PD1)
#define DEBUGPIN_ON PORTD|=(1 << PD1)

// Leds at Port B
#define LED1 (1 << 5)
#define LED1_ON  PORTB|= LED1
#define LED1_OFF PORTB&=~LED1

#define LED3 (1 << 2)
#define LED3_ON  PORTB|= LED3
#define LED3_OFF PORTB&=~LED3

#define LED4 (1 << 4)
#define LED4_ON  PORTB|= LED4
#define LED4_OFF PORTB&=~LED4

// LEDs at port D
#define LED2 (1 <<6)
#define LED2_ON  PORTD|= LED2
#define LED2_OFF PORTD&=~LED2

#define INIT_LED { DDRB|=LED1|LED3|LED4;DDRD|=LED2;}

void delay(uint16_t n)
{
	uint32_t k;
	for (k=1;k<(200*n);k++);  /* Verzgerung */
}
/*******************************************************************************
	Kommandos zur Ansteuerung der Test-LED
*******************************************************************************/
void toggle_led(void)	
{
	static uint8_t k=0;
	if (k==0)
	{
		LED1_ON;
		k=1;
	}
	else
	{
		LED1_OFF;
		k=0;
	}
			
}

void lauflicht(uint8_t n)	
{
	uint8_t k,i;

	for(i=0;i<n;i++)
	{
		for(k=0;k<=3;k++)
		{
			LED1_OFF;
			LED3_OFF;
			LED4_OFF;

			if(k==0) LED1_ON;
			if(k==1) LED3_ON;
			if(k==2) LED4_ON;
			delay(50);
		}
	}
}

void blink_led(uint8_t n)
{
	do 
	{
		delay(100);
		toggle_led();
	}while(n--);
	LED1_OFF;
}

void Init (void)
{
	// Debug Pin
	DDRD=(1 << PD1);
	DEBUGPIN_ON;
	DEBUGPIN_OFF;

	// I/O Ports
	DDRB = (1 << PB1) | (1 << PB2);

	// TIMER2: set OC2 to output
	DDRB|=( 1 << PB3);
	
	// for FAST PWM (8-Bit PWM) on OC1A 
	TCCR1A = (1 << WGM10) | (1 << COM1A1) ;

	// tmr1 running on MCU clock/8 
	//TCCR1B = (1 << CS11);
	// tmr1 running on MCU full speed clock 
	TCCR1B = (1 << WGM12) |(1 << CS10);
	// tmr1 using external clock on T1 ( Pin11 )	
//***	 disable next line for debugging, this will enable T1 Int
#ifndef DEBUG
	TCCR1B |= (1 << CS10)|(1 << CS11)|(1 << CS12);
#endif

#ifdef ATMEGA8
	// TIMER2 used as clock devider FCPU/2
	// TIMER2: CTC Mode
	TCCR2 = (1 << WGM21);
	// TIMER2: clock no prescaling
	TCCR2 |= (1 << CS20);
	// TIMER2: toogle OC2 on compare match
	TCCR2 |= (1 << COM20);
	// TIMER2: set compare value
	OCR2=0;

	// interrupt mask register: enable timer1 overflow 
	TIMSK |= (1 << TOV1);

	// prepare RS232 
	UCSRA = 0x00;
	UCSRB = 0x00;	
	UCSRC = 0x86; // No Parity | 1 Stop Bit | 8 Data Bit
	//UBRR0L = 0xCF; // 2400bps @ 8.00MHz
	//UBRRL = 51; // 9600bps @ 8.00MHz
	// UBRR0L = 25; // 19200bps @ 8.00MHz	

	//UCSR0B =  (1<<TXEN0) ; // enable transmitter
	UCSRB |= (1<<RXEN0) ; // enable receiver

#endif
#ifdef ATMEGA168

	// TIMER2 used as clock devider FCPU/2
	// TIMER2: CTC Mode
	//TCCR2 = (1 << WGM21);
	TCCR2A = (1 << WGM21);
	// TIMER2: clock no prescaling
	//TCCR2 |= (1 << CS20);
	TCCR2B |= (1 << CS20);
	// TIMER2: toogle OC2 on compare match
	TCCR2A |= (1 << COM2A0);
	// TIMER2: set compare value
	OCR2A=0;

	TIMSK1 |= (1 << TOIE1);

	// prepare RS232 
	UCSR0A = 0x00;
	UCSR0B = 0x00;	
	UCSR0C = 0x86; // No Parity | 1 Stop Bit | 8 Data Bit

	//UBRR0L = 0xCF; // 2400bps @ 8.00MHz
	UBRR0L = 103; // 4800bps @ 8.00MHz, 9600@16MHz
	//UBRR0L = 51; // 9600bps @ 8.00MHz
	// UBRR0L = 25; // 19200bps @ 8.00MHz

	UCSR0B =  (1<<TXEN0) ; // enable transmitter
	UCSR0B |= (1<<RXEN0) ; // enable receiver

#endif

	sei();
}
uint8_t getchar(void)
{
#ifdef ATMEGA8
#define SERIAL_RECEIVED (UCSRA & 0x80)
#define SERIAL_DATA UDR
#endif

#ifdef ATMEGA168
#define SERIAL_RECEIVED (UCSR0A & 0x80)
#define SERIAL_DATA UDR0
#endif

//	while(!(UCSRA & TXC)); // wait for empty transmit buffer to avoid trasmitter LED light 

	//UCSR0B |= (1<<RXEN0) ; // enable receiver
	while(!SERIAL_RECEIVED); // wait for received byte
	return SERIAL_DATA;
}
/************************************************************************

	interrupt routine 
	
	- calculate waverform phases
	- calculate waveforms
	- set PWM output
	- increase system timer  
	
************************************************************************/
SIGNAL (SIG_OVERFLOW1)
{
	static uint16_t phase0,phase1,phase2;
	static int16_t temp,temp1;
	static uint8_t k,flag=0;
	static uint8_t timer;
	static uint16_t noise=0xAA,noise8;
	static uint16_t sig0,sig1,sig2;
	static uint16_t tempphase;

	//DEBUGPIN_ON; // Pin to measure processing time

	flag^=1;
	if(flag)
	{
		//DEBUGPIN_ON; // Pin to measure processing time
		// Voice1
		tempphase=phase0+freq_coefficient[0]; //0.88us
		if(Sid.block.voice1.ControlReg&NOISE)
		{				
			if((tempphase^phase0)&0x4000) sig0=noise8*envelope[0];			
		}
		else
		{
			if(Sid.block.voice1.ControlReg&RINGMOD)
			{				
				if(phase2&0x8000) sig0=envelope[0]*-wave0[phase0>>8];
				else sig0=envelope[0]*wave0[phase0>>8];
			}
			else sig0=envelope[0]*wave0[phase0>>8]; //2.07us
		}
		phase0=tempphase;

		// Voice2
		tempphase=phase1+freq_coefficient[1]; //0.88us
		if(Sid.block.voice2.ControlReg&NOISE)
		{				
			if((tempphase^phase1)&0x4000) sig1=noise8*envelope[1];
		}
		else
		{
			if(Sid.block.voice2.ControlReg&RINGMOD)
			{				
				if(phase0&0x8000) sig1=envelope[1]*-wave1[phase1>>8];
				else sig1=envelope[1]*wave1[phase1>>8];
			}
			else sig1=envelope[1]*wave1[phase1>>8]; //2.07us
		}
		phase1=tempphase;

		// Voice3
		tempphase=phase2+freq_coefficient[2]; //0.88us
		if(Sid.block.voice3.ControlReg&NOISE)
		{				
			if((tempphase^phase2)&0x4000) sig2=noise8*envelope[2];
		}
		else
		{
			if(Sid.block.voice3.ControlReg&RINGMOD)
			{				
				if(phase1&0x8000) sig2=envelope[2]*-wave2[phase2>>8];
				else sig2=envelope[2]*wave2[phase2>>8];
			}
			else sig2=envelope[2]*wave2[phase2>>8]; //2.07us
		}
		phase2=tempphase;
	}
	else
	{		
		DEBUGPIN_ON; // Pin to measure processing time

		//		temp=sig0+sig1+sig2;
		
		// voice filter selection
		temp=0; // direct output variable
		temp1=0; // filter output variable
		if(Sid.block.RES_Filt&FILT1) temp1+=sig0;
		else temp+=sig0;
		if(Sid.block.RES_Filt&FILT2) temp1+=sig1;
		else temp+=sig1;
		if(Sid.block.RES_Filt&FILT3) temp1+=sig2;
		else if(!(Sid.block.Mode_Vol&VOICE3OFF))temp+=sig2; // voice 3 with specal turn off bit
	
		//filterOutput = IIR2((struct IIR_filter*)&filter04_06, filterInput);
		//IIR2(filter04_06, temp1);
		k=(temp>>8)+128;
		k+=temp1>>10; // not real filter implemeted yet
		
		OCR1A=k; // Output to PWM
		
		//DEBUGPIN_ON; // Pin to measure processing time
		
		// noise generator
		for(k=1;k<2;k++)
		{
			temp1 = noise;
			noise=noise << 1;
	
			temp1 ^= noise;
			if ( ( temp1 & 0x4000 ) == 0x4000 ) 
			{ 
				noise |= 1;
			}
		}
		noise8=noise>>6;
#ifdef NOISEOFF
		noise8=0;
#endif

		// serial receiver

		if(SERIAL_RECEIVED) // test if byte received
		{
			k=SERIAL_DATA;
			
			// we can either receive a register address
			// or a register value
			// Serial_receiver_bytes[0]=register address
			// Serial_receiver_bytes[1]=register value
			// if register address>0 ==> error NUMREGISTERS
			if(!((Serial_receiver_counter==0)&&(k>NUMREGISTERS))) // if register address ok
			{
				Serial_receiver_bytes[Serial_receiver_counter]=k;
				Serial_receiver_counter++;
				if(Serial_receiver_counter>2) Serial_receiver_counter=0;
			}			
		}	
	}

	timer--;
	if(timer==0) 
	{
		mstimer++; // increase millisecond timer
		// mstimerrate=0.992 milli seconds @FCPU16MHz 
		timer=31;
	}
	DEBUGPIN_OFF; // Pin to measure time

}
/************************************************************************/

uint8_t get_wavenum(struct Voice *voice)
{
	uint8_t n;

	if(voice==&Sid.block.voice1) n=0;
	if(voice==&Sid.block.voice2) n=1;
	if(voice==&Sid.block.voice3) n=2;
	
	return n;
}

void setfreq(struct Voice *voice,uint16_t freq)
{
	uint32_t templong;
	uint8_t n;

	n=get_wavenum(voice);
	
	templong=freq;
	freq_coefficient[n]=templong*65536/FS;
}

void init_waveform(struct Voice *voice)
{
	uint16_t n;
	int16_t val;
	uint8_t wavetype=voice->ControlReg;
	int8_t *wave_array;

	n=get_wavenum(voice);

	if(n==0) wave_array=wave0;
	if(n==1) wave_array=wave1;
	if(n==2) wave_array=wave2;

	val=-128;

	for(n=0;n<256;n++)
	{
		*wave_array=0xFF;

		if(wavetype&SAWTOOTH)  
		{
			*wave_array&=val;
		}

		if(wavetype&TRIANGLE) 
		{
				if(n&0x80) *wave_array&=((n^0xFF)<<1)-128;
				else *wave_array&=(n<<1)-128;
		}

		if(wavetype&RECTANGLE) 
		{
			if(n>(voice->PW >> 4)) // SID has 12Bit pwm, here we use only 8Bit
			{
				*wave_array&=127;
			}
			else *wave_array&=-127;
		}
		
		// if(wavetype&NOISE) *wave_array&=zufall();
		// noise has to be online calculated

		val++;
		wave_array++;
	}
}
//#define MAXLEVEL ( 0xFFFF / OSCILLATORS )
#define MAXLEVEL  19000
#define SUSTAINFAKTOR ( MAXLEVEL / 15 )

void setenvelope(struct Voice *voice)
{
	uint8_t n;
	
	n=get_wavenum(voice);
	attackdecay_flag[n]=TRUE;

//	t_attack[n]=AttackRate[voice->AttackDecay>>4];
//	t_decay[n]=DecayReleaseRate[voice->AttackDecay&0x0F];
//	t_release[n]=DecayReleaseRate[voice->SustainRelease&0x0F];
	level_sustain[n]=(voice->SustainRelease>>4)*SUSTAINFAKTOR;
	
//	m_attack[n]=MAXLEVEL/t_attack[n];
	m_attack[n]=MAXLEVEL/AttackRate[voice->AttackDecay>>4];
//	m_decay[n]=(MAXLEVEL-level_sustain[n]*SUSTAINFAKTOR)/t_decay[n];
	m_decay[n]=(MAXLEVEL-level_sustain[n]*SUSTAINFAKTOR)/DecayReleaseRate[voice->AttackDecay&0x0F];

//	m_release[n]=(level_sustain[n]*SUSTAINFAKTOR)/t_release[n];
	m_release[n]=(level_sustain[n])/DecayReleaseRate[voice->SustainRelease&0x0F];
}
/************************************************************************
	
	uint8_t set_sidregister(uint8_t regnum, uint8_t value)

	The registers of the virtual SID are set by this routine.
	For some registers it is necessary to transform the SID-register
	values to some internal settings of the emulator. 
	To select this registers and to start the calculation, the switch/
	case statement is used.
	For instance: If setting the SID status register, new waveforms in 
	the waveform array are calculated.

	4.2007 ch

************************************************************************/
uint8_t set_sidregister(uint8_t regnum, uint8_t value)
{
	//uint16_t temp;

	if(regnum>NUMREGISTERS-1) return 1;
	Sid.sidregister[regnum]=value;

	switch(regnum)
	{
		//voice1
		case 1:
		{
			//temp=(Sid.sidregister[0]+(Sid.sidregister[1]<<8))/17;
			//setfreq(&Sid.block.voice1,temp);

			freq_coefficient[0]=(Sid.sidregister[0]+(Sid.sidregister[1]<<8))/4;

		}break;
		case 3: init_waveform(&Sid.block.voice1);break;
		case 4: init_waveform(&Sid.block.voice1);break;
		case 5: setenvelope(&Sid.block.voice1);break;
		case 6: setenvelope(&Sid.block.voice1);break;
		
		//voice2
		case 8:
		{
			//temp=(Sid.sidregister[7]+(Sid.sidregister[8]<<8))/17;
			//setfreq(&Sid.block.voice2,temp);
			freq_coefficient[1]=(Sid.sidregister[7]+(Sid.sidregister[8]<<8))/4;
		}break;
		case 10: init_waveform(&Sid.block.voice2);break;
		case 11: init_waveform(&Sid.block.voice2);break;
		case 12: setenvelope(&Sid.block.voice2);break;
		case 13: setenvelope(&Sid.block.voice2);break;		
		
		//voice3
		case 15:
		{
			//temp=(Sid.sidregister[14]+(Sid.sidregister[15]<<8))/17;
			//setfreq(&Sid.block.voice3,temp);
			freq_coefficient[2]=(Sid.sidregister[14]+(Sid.sidregister[15]<<8))/4;
		}break;
		case 17: init_waveform(&Sid.block.voice3);break;
		case 18: init_waveform(&Sid.block.voice3);break;
		case 19: setenvelope(&Sid.block.voice3);break;
		case 20: setenvelope(&Sid.block.voice3);break;			
	}	
	return 0;
}
/************************************************************************

	main loop

	- calculate envelopes
	- process scheduler

************************************************************************/
int main(void)
{
	int16_t amp[OSCILLATORS];
	uint16_t schedule_timer=0,temp=0;
	uint8_t n,flag;
	uint8_t controll_regadr[3]={4,11,18};
	uint16_t soundindex=0;

	Init();
	INIT_LED;

	blink_led(120);
	lauflicht(3);
	
	UDR0 = 0x48;
	UDR0 = 0x41;
	UDR0 = 0x4c;
	UDR0 = 0x4c;
	UDR0 = 0x4f;
	UDR0 = 0x21;
	

	attackdecay_flag[0]=TRUE;
	attackdecay_flag[1]=TRUE;
	attackdecay_flag[2]=TRUE;
		
	//initialize SID-registers
	
	Sid.sidregister[6]=0xF0;
	Sid.sidregister[13]=0xF0;
	Sid.sidregister[20]=0xF0;
	setenvelope(&Sid.block.voice1);
	setenvelope(&Sid.block.voice2);
	setenvelope(&Sid.block.voice3);
	
	// set all amplitudes to zero
	for(n=0;n<OSCILLATORS;n++) amp[n]=0;

	while(1)
	{
		if(Sid.sidregister[VOICE1+CONTROLREG]&GATE)LED1_ON;
		else LED1_OFF;
		if(Sid.sidregister[VOICE2+CONTROLREG]&GATE)LED3_ON;
		else LED3_OFF;
		if(Sid.sidregister[VOICE3+CONTROLREG]&GATE)LED4_ON;
		else LED4_OFF;

		//DEBUGPIN_ON; // Pin to measure processing time
		
		// calculate envelopes
		// if gate is ONE then the attack,decay,sustain cycle begins
		// if gate switches to zero the sound decays
		for(n=0;n<OSCILLATORS;n++)
		{
			if(Sid.sidregister[controll_regadr[n]]&GATE) // if gate set then attack,decay,sustain
			{
				if(attackdecay_flag[n]) 
				{	// if attack cycle
					amp[n]+=m_attack[n];
					if(amp[n]>MAXLEVEL)
					{
						amp[n]=MAXLEVEL;
						attackdecay_flag[n]=FALSE; // if level reached, then switch to decay
					}
				}
				else // decay cycle
				{
					if(amp[n]>level_sustain[n])
					{
						amp[n]-=m_decay[n];
						if(amp[n]<level_sustain[n]) amp[n]=level_sustain[n];
					}

				} 
			}
			else // if gate flag is not set then release
			{
				attackdecay_flag[n]=TRUE; // at next attack/decay cycle start wiht attack
				if(amp[n]>0)
				{
					amp[n]-=m_release[n];
					if(amp[n]<0) amp[n]=0;
				}			
			}
			envelope[n]=amp[n]>>8;
		}
		//DEBUGPIN_OFF; // Pin to measure processing time
		
		//DEBUGPIN_ON; // Pin to measure processing time#
		
		

		if(schedule_timer==0)
		{
//			schedule_timer=MAXTIME;
			
			flag=TRUE;
			while(flag)
			{
				n=pgm_read_byte_near(&sound[soundindex]);
				soundindex++;
				
				if(n<NUMREGISTERS)
				{
					set_sidregister(n,pgm_read_byte_near(&sound[soundindex]));
					soundindex++;
				}
				else 
				{
					if(n==0xF0) // get lower part of pause to next tone
					{
						schedule_timer=pgm_read_byte_near(&sound[soundindex]);
						soundindex++;
						flag=TRUE;	
					}
					if(n==0xF1) // get higher part of pause to next tone
					{
						schedule_timer|=pgm_read_byte_near(&sound[soundindex])<<8;
						soundindex++;
						flag=FALSE;	
					}
					if(n==0xF2) // replay command
					{
						soundindex=0;
					}
					if(n==0xF3) // one byte delay comand
					{
						schedule_timer=pgm_read_byte_near(&sound[soundindex]);
						soundindex++;
						flag=FALSE;	
					}
					flag=FALSE;
					if(n==0xFF) 
					{
							cli(); // stop sound interrupt
							while(1); // Stop Programm execution
					}
				}
			}
			

		}
		else schedule_timer--;
		
		
		

		

		//while(Serial_receiver_counter==2);
		if(Serial_receiver_counter==2) 
		{
			set_sidregister(Serial_receiver_bytes[0],Serial_receiver_bytes[1]);
			Serial_receiver_counter=0;
			toggle_led();
			UDR0 = Serial_receiver_bytes[0];
			UDR0 = Serial_receiver_bytes[1];
			
		}

//		DEBUGPIN_OFF; // Pin to measure processing time

//		DEBUGPIN_ON; // Pin to measure remaining processing time
		while(temp==mstimer); // wait for millisecond multiple
		temp=mstimer;
//		DEBUGPIN_OFF;
	}

}
